#!/usr/local/bin/perl
use strict;
use lib qw(./lib);
use Net::Kubernetes;
require Net::Kunbernetes::Exception::NotFound;
use Data::Dumper;
use syntax 'try';

my $kube = Net::Kubernetes->new(url=>'http://172.18.8.101:8080');

my $ns = $kube->get_namespace('default');

my $sec = $ns->build_secret('testing', { 'sshpubkeypub' => '/home/dave/.ssh/id_rsa.pub', 'otherthingjson' => {type=>'JSON', value=>{text=>'Ha Ha'}} });





